package org.freecompany.brimstone.main;

import java.io.PrintStream;
import java.util.Arrays;

public class Formatter {

	protected final static String[] SUFFIXES = new String[] { "B", "KB", "MB", "GB", "TB"};
	protected final static int length = 35;

	protected boolean showTotal;
	protected boolean showCurrent;

	protected final PrintStream stream;
	protected String name;
	protected long total;
	protected long current;

	/**
	 * If stream is null then we use system output, but don't provide realtime updates
	 * as certain applications (Ant) hijack {@link System#out} and provide one that does
	 * not handle carriage returns gracefully.
	 */
	public Formatter( final String name, final PrintStream stream) {
		this.name = name;
		this.stream = stream;
		if ( stream != null) showCurrent( true);
	}
	
	public void showTotal( final boolean showTotal) { this.showTotal = showTotal; }
	public void showCurrent( final boolean showCurrent) { this.showCurrent = showCurrent; }
	
	public void setTotal( final long total) {
		this.total = total;
		if ( total > 0) showTotal( true);
		
		if ( stream == null) System.out.println( format());
		else stream.print( "\r" + format());
	}

	protected String format( final float number, final int count) {
		if ( number < 100) return String.format( "%1$.1f", number) + SUFFIXES[ count];
		else if ( number < 1000) return String.format( "%1$.0f", number) + SUFFIXES[ count];
		else return format( number / 1024, count + 1);
	}

	public void read( final int read) {
		current += read;
		if ( stream != null) stream.print( "\r" + format());
	}

	public void finish() {
		if ( stream != null) stream.print( "\n");
	}

	protected String format() {
		StringBuilder string = new StringBuilder();
		if ( showTotal || showCurrent) {
			string.append( "[");
			if ( showCurrent) string.append( format( current, 0));
			if ( showCurrent && showTotal) string.append( "/");
			if ( showTotal) string.append( format( total, 0));
			string.append( "]");
		}
		
		final int size = length - string.length();
		string.insert( 0, String.format( "%1$-" + size + "s", name.length() > ( size - 3) ? name.substring( 0, size - 3) + "..." : name));

		if ( showCurrent && showTotal) {
			final char[] buffer = new char[ 20];
			final int progress = ( int) ((( float) current / total) * buffer.length);
			Arrays.fill( buffer, progress, buffer.length, ' ');
			Arrays.fill( buffer, 0, progress, '*');
			string.append( " |" + new String( buffer) + "|");
		}
		return string.toString();
	}
}
